/*
  EASYROBOT2 + FISHINO + ROBOREMO
  
  This sketch use ROBOREMO app for control Fishino+easyrobot2_schield
  send/receive UDP packet over Wifi

  Mirco Segatello 2016   
  Arduino IDE 1.6.10
*/


//#include <Flash.h>
//#include <FishinoUdp.h>
//#include <FishinoSockBuf.h>
#include <Fishino.h>
#include <SPI.h>


//////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
// CONFIGURATION DATA    -- ADAPT TO YOUR NETWORK !!!
// DATI DI CONFIGURAZIONE -- ADATTARE ALLA PROPRIA RETE WiFi !!!

// here pur SSID of your network
// inserire qui lo SSID della rete WiFi
#define MY_SSID "MY_SSID"

// here put PASSWORD of your network. Use "" if none
// inserire qui la PASSWORD della rete WiFi -- Usare "" se la rete non ï¿¨ protetta
#define MY_PASS "MY_PASS"

// here put required IP address of your Fishino
// comment out this line if you want AUTO IP (dhcp)
// NOTE : if you use auto IP you must find it somehow !
// inserire qui l'IP desiderato per il fishino
// commentare la linea sotto se si vuole l'IP automatico
// nota : se si utilizza l'IP automatico, occorre un metodo per trovarlo !
#define IPADDR  192, 168,   0, 242
// local port to listen on
// porta UDP sulla quale Fishino attende i pacchetti
unsigned int localPort = 2390;


//                    END OF CONFIGURATION DATA                      //
//                       FINE CONFIGURAZIONE                         //
///////////////////////////////////////////////////////////////////////

// define ip address if required
// NOTE : if your network is not of type 255.255.255.0 or your gateway is not xx.xx.xx.1
// you should set also both netmask and gateway
#ifdef IPADDR
IPAddress ip(IPADDR);
#endif


// buffer to hold incoming packet
char cmd[255];

// for use Easyrobot2
long lasttime;
int steer=0, speed=0, isConnect=0;
String sendData;
// valori di tensione per calcolare il livello in %
// valori calibrati su 4 celle NiMh
const float VbatMIN = 4.0; //tensione a batteria scarica
const float VbatMAX = 6.5; //tensione a batteria carica
float distanza;


// the UDP client/server
FishinoUDP Udp;

void printWifiStatus()
{
  
  // print the SSID of the network you're attached to:
  Serial.print("SSID: ");
  Serial.println(Fishino.SSID());

  // print your WiFi shield's IP address:
  IPAddress ip = Fishino.localIP();
  Serial << F("IP Address: ");
  Serial.println(ip);
  Serial << F("Port: ");
  Serial.println(localPort);
  
  // print the received signal strength:
  long rssi = Fishino.RSSI();
  Serial << F("signal strength (RSSI):");
  Serial.print(rssi);
  Serial << F(" dBm\n");
}

// setup code
// codice di inizializzazione
void setup()
{
  // Initialize serial and wait for port to open
  Serial.begin(115000);  //for debug
  
  // only for Leonardo needed
  while (!Serial)
    ;

  // initialize easyrobot
  initialize();
    
  // initialize SPI
  SPI.begin();
  SPI.setClockDivider(SPI_CLOCK_DIV2);
  
  // reset and test WiFi module
  while(!Fishino.reset())
    Serial << F("Fishino RESET FAILED, RETRYING...\n");
  Serial << F("Fishino WiFi RESET OK\n");

  // go into station mode
  // imposta la modalità  stazione
  Fishino.setMode(STATION_MODE);

  // try forever to connect to AP
  // tenta la connessione finchà¨ non riesce
  Serial << F("Connecting to AP...");
  while(!Fishino.begin(MY_SSID, MY_PASS))
  {
    Serial << ".";
    delay(2000);
  }
  Serial << "OK\n";
  
  
  // setup IP or start DHCP client
  // imposta l'IP statico oppure avvia il client DHCP
#ifdef IPADDR
  Fishino.config(ip);
#else
  Fishino.staStartDHCP();
#endif

  // wait till connection is established
  Serial << F("Waiting for IP...");
  while(Fishino.status() != STATION_GOT_IP)
  {
    Serial << ".";
    delay(500);
  }
  Serial << "OK\n";

  
  // print connection status on serial port
  // stampa lo stato della connessione sulla porta seriale
  printWifiStatus();

  // starts listening on local port
  // inizia l'ascolto dei pacchetti UDP alla porta specificata
  Serial << F("Starting connection\n\n");
  Udp.begin(localPort);
}

void loop()
{
  
  // if there's data available, read a packet
  // se ci sono dati in arrivo, li stampa
  int packetSize = Udp.parsePacket();
  if (packetSize)
  {
    /*
    Serial << F("Received packet of size ");
    Serial << packetSize << "\n";
    Serial << F("From ");
    IPAddress remoteIp = Udp.remoteIP();
    Serial << remoteIp;
    Serial << F(", port ");
    Serial << Udp.remotePort() << "\n";
    */
    // read the packet into packetBufffer
    // legge il pacchetto nel buffer
    int len = Udp.read(cmd, 255);
    if (len > 0) cmd[len] = 0;

    isConnect=1;  //if receive packets is connected!

    // debug for received data
    //Serial << F("Receive: ");
    //Serial.print(cmd);

     // Parse incoming speed data
     if(cmd[0]=='s' && cmd[1]=='p' && cmd[2]=='e' && cmd[3]=='e' && cmd[4]=='d' && cmd[5]==' ') 
     {  
       speed = 0;
       if (cmd[6]=='-')
       {
            for(int i=7; cmd[i]!='\n'; i++) {
             speed = speed*10 + cmd[i]-'0';
           }  
           speed = -speed; 
       }
       else
       {
           for(int i=6; cmd[i]!='\n'; i++) {
             speed = speed*10 + cmd[i]-'0';
           }  
       }   
       //Serial.println(speed);
     }

     // Parse incoming steer data
     if(cmd[0]=='s' && cmd[1]=='t' && cmd[2]=='e' && cmd[3]=='e' && cmd[4]=='r' && cmd[5]==' ') 
     {  
       steer = 0;
       if (cmd[6]=='-')
       {
           for(int i=7; cmd[i]!='\n'; i++) {
             steer = steer*10 + cmd[i]-'0';
           }  
           steer = -steer; 
       }
       else
       {
           for(int i=6; cmd[i]!='\n'; i++) {
             steer = steer*10 + cmd[i]-'0';
           }   
       }   
     }

     // do command
     turn(steer, speed);
  }

  
    // invia il livello della batteria all'indirizzo IP e alla porta da cui proviene la richiesta
    if (millis() > lasttime+1000)
    {
        //Verifica ogni secondo se manca segnale
        if (isConnect==0) 
          turn(0,0);
        isConnect=0;
                
        //send battery level
        char tmp[4];
        int BatLevel=100*(VBat()-VbatMIN)/(VbatMAX-VbatMIN);
        BatLevel = constrain(BatLevel, 0, 100);
        sprintf(tmp,"%d",BatLevel);  
        String VbatString(tmp);
        sendData= "VBAT " + VbatString + "%\n";

        //Serial.print("Send Reply:");
        //Serial.print(sendData);
        Udp.beginPacket(Udp.remoteIP(), localPort);
        Udp.print(sendData);
        Udp.endPacket();
        lasttime=millis();
    }

/*
    distanza = CNY70(1);
    Serial.print(distanza);
    distanza = CNY70(2);
    Serial.print(" - ");
    Serial.print(distanza);
    distanza = CNY70(3);
    Serial.print(" - ");
    Serial.print(distanza);
    distanza = distance();
    Serial.print(" - ");
    Serial.print(distanza);
    Serial.println("cm");
    */
}


